# call this module via:  node 'name' {include ossec::server}
# replace server with  local or agent  depending on the type of ossec install you
# want to perform.

class ossec {

    define download_file(
            $site="",
            $cwd="",
            $creates="",
            $require="",
            $user="") {                                                                                         

            file {"${cwd}/${name}":
            ensure => present,
            source => "puppet:///modules/ossec/$name",
            }
    }        


    class install inherits packages::openssldev {

        $ossecversion = "ossec-hids-2.7" 
        $ossecfile = "$ossecversion.tar.gz"
        $workdir = "/usr/local/src"


        file { "/usr/local/src":    
            ensure  => directory,
            owner => root,
            group => root,
            mode => 760, 
        }

        download_file {"${ossecfile}":                                                                                                                              
           site => "http://www.ossec.net/files/",
           cwd => "${workdir}",                                                                            
           creates => "${workdir}/$name",                                                                  
           require => File["/usr/local/src"],                                                                  
           user => root,                                                                                                              
       }

        exec {"extract-ossec":
           cwd => "${workdir}",
           command => "/bin/tar xzf ${ossecfile}",
           creates => "${workdir}/${ossecversion}",
           require => [ Download_file["${ossecfile}"], Package["$openssldev"] ],
           user => root,
        }

    }


    class server inherits ossec::install {

        $ossectype = "server"

        file {"ossecvars":
            path => "${workdir}/${ossecversion}/etc/preloaded-vars.conf",
            ensure => present,
            content => template("ossec/preloaded-vars.conf-${ossectype}"), 
            require => Exec["extract-ossec"],

        }

        file {"/var/ossec":
          ensure => directory,
        }

        exec {"install-ossec":
            cwd     => "${workdir}/${ossecversion}",
            command => "${workdir}/${ossecversion}/install.sh",
            creates => "/var/ossec/etc",
            user    => root,
            require => [File["ossecvars"], File["/var/ossec"], ],
        }

        service { "ossec":
            enable  => true,
            ensure  => running,
            require => [Exec["install-ossec"],],
        }

        user{'ossec':
            ensure => present,
            groups => 'ossec',
        }

        group{'ossec':
            ensure  => present,
            require => User['ossec'],
        }

        # manage ossec.conf file
        file { "ossec.conf":
            path    => "/var/ossec/etc/ossec.conf",
            ensure  => present, 
            owner   => root, group => ossec, mode => 550,
            content => template("ossec/ossec-conf-${ossectype}.erb"),
            require => [ Exec["install-ossec"], File["/var/ossec"], Group['ossec'], ], 
        } 

        # manage the /var/ossec/rules 
        file { "ossec-rules":
            path => "/var/ossec/rules",
            checksum => "mtime",
            ensure  => directory, owner => root, group => ossec, mode => 550, 
            source  => "puppet:///modules/ossec/ossec-rules",
            recurse => true,
            ignore  => [ ".svn" ],
            require => [ File["ossecvars"], Exec["install-ossec"],]
        }

        exec {ossec-restart:
            command => "/var/ossec/bin/ossec-control restart",
            subscribe => File[ "ossec.conf" , "ossec-rules" ],
            refreshonly => true,  # Only run command if monitored files change       
        }

        # ossec-authd
        file{"/var/ossec/etc/sslmanager.key":
          ensure  => present,
          owner   => ossec,
          require => [ Exec["sslmanager-key"], Exec["sslmanager-cert"],],
        }

        file{"/var/ossec/etc/sslmanager.cert":
          ensure  => present,
          owner   => ossec,
          require => Exec["sslmanager-cert"],
        }

        exec{"sslmanager-key":
          cwd     => "/var/ossec",
          command => "/usr/bin/openssl genrsa -out /var/ossec/etc/sslmanager.key 2048",
          creates => "/var/ossec/etc/sslmanager.key",
          user    => root,
        }

        exec{"sslmanager-cert":
          cwd     => "/var/ossec",
          command => "/usr/bin/openssl req -new -x509 -key /var/ossec/etc/sslmanager.key -out /var/ossec/etc/sslmanager.cert -days 365 -subj /CN=hhvmwebintprd01/O=ecls/",
          creates => "/var/ossec/etc/sslmanager.cert",
          user    => root,
          require => Exec["sslmanager-key"],
        }

        file{"/etc/init.d/ossec-authd":
          ensure => present,
          owner  => "root",
          group  => "root",
          mode   => "0755",
          source => "puppet:///modules/ossec/ossec-authd",
        }

        service{"ossec-authd":
          enable  => true,
          ensure  => running,
          require => [File["/var/ossec/etc/sslmanager.cert"],Exec["install-ossec"],],
        }
    }


    class local inherits ossec::install {

        $ossectype = "local"

        file {"ossecvars":
            path => "${workdir}/${ossecversion}/etc/preloaded-vars.conf",
            ensure => present,
            content => template("ossec/preloaded-vars.conf-${ossectype}"), 
            require => Exec["extract-ossec"],

        }

        exec {"install-ossec":
            cwd => "${workdir}/${ossecversion}",
            command => "${workdir}/${ossecversion}/install.sh",
            creates => "/var/ossec/etc",
            user => root,
            require => File["ossecvars"],
        }

        service { "ossec":
            enable  => true,
            ensure  => running,
            require => Exec["install-ossec"],
        }

        user{'ossec':
            ensure => present,
            groups => 'ossec',
        }

        group{'ossec':
            ensure  => present,
            require => User['ossec'],
        }

        # manage ossec.conf file
        file { "ossec.conf":
            path    => "/var/ossec/etc/ossec.conf",
            ensure  => present,
            owner   => root,
            group   => ossec,
            mode    => 550,
            content => template("ossec/ossec-conf-${ossectype}.erb"),
            require => Service["ossec"],
        } 

        # manage the /var/ossec/rules 
        file { "ossec-rules":
            path => "/var/ossec/rules",
            checksum => "mtime",
            ensure  => directory, owner => root, group => ossec, mode => 550, 
            source  => "puppet://$server/ossec/ossec-rules",
            recurse => true,
            ignore  => [ ".svn" ],
        }

        exec {ossec-restart:
            command => "/var/ossec/bin/ossec-control restart",
            subscribe => File[ "ossec.conf" , "ossec-rules" ],
            refreshonly => true,  # Only run command if monitored files change       
        }

    }


    class agent inherits ossec::install {    
        $ossectype = "agent"
        $ossecserver= "10.128.20.55"

        file {"ossecvars":
            path => "${workdir}/${ossecversion}/etc/preloaded-vars.conf",
            ensure => present,
            content => template("ossec/preloaded-vars.conf-${ossectype}"), 
            require => Exec["extract-ossec"],

        }

        exec {"install-ossec":
            cwd     => "${workdir}/${ossecversion}",
            command => "${workdir}/${ossecversion}/install.sh",
            creates => "/var/ossec/etc",
            user    => root,
            require => File["ossecvars"],
        }

        exec { "client.keys":
          command   => "/var/ossec/bin/agent-auth -m $ossecserver  -A $::fqdn -D /var/ossec/",
          creates   => "/var/ossec/etc/client.keys",
          require   => Exec["install-ossec"],
        }

        file{"/var/ossec/etc/client.keys":
          ensure  => present,
          require => Exec["client.keys"],
        }

        service { "ossec":
          enable     => true,
          ensure     => running,
          hasrestart => true,
          hasstatus  => true,
          status     => '/etc/init.d/ossec status | grep "ossec-agentd is running"',
          require    => Exec["install-ossec"],
        }

        user{'ossec':
            ensure => present,
        }

        group{'ossec':
            ensure  => present,
            require => User['ossec'],
        }

        # manage ossec.conf file
        file { "ossec.conf":
            path    => "/var/ossec/etc/ossec.conf",
            ensure  => present,
            owner   => root,
            group   => ossec,
            mode    => 550,
            content => template("ossec/ossec-conf-${ossectype}.erb"),
            require => [ Exec["install-ossec"], Group['ossec'], ],
        } 

        exec {"ossec-restart":
            command     => "/var/ossec/bin/ossec-control restart",
            subscribe   => [File["ossec.conf"],File["/var/ossec/etc/client.keys"],],
            refreshonly => true,
        }
    }
}
